/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.perstatistics.impl;

import cn.hutool.core.util.StrUtil;
import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.yiring.app.domain.perstatistics.PersonnelStatistics;
import com.yiring.app.domain.perstatistics.PersonnelStatisticsRepository;
import com.yiring.app.excel.perstatistics.PersonnelStatisticsExportExcel;
import com.yiring.app.param.perstatistics.PersonnelStatisticsQueryParam;
import com.yiring.app.service.perstatistics.PersonnelStatisticsService;
import com.yiring.app.vo.perstatistics.PersonnelStatisticsVo;
import com.yiring.common.core.Result;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author tzl
 * @version 1.0
 * @description:
 * @date 2022/5/15 9:28
 */
@Slf4j
@Transactional(rollbackFor = RuntimeException.class)
@Service
public class PersonnelStatisticsServiceImpl implements PersonnelStatisticsService {

    @Resource
    PersonnelStatisticsRepository personnelStatisticsRepository;

    @Override
    public Result<PageVo<PersonnelStatisticsVo>> pagePerSta(
        PersonnelStatisticsQueryParam personnelStatisticsQueryParam,
        PageParam pageParam
    ) {
        //分页
        Pageable pageable = PageRequest.of(pageParam.getPageNo() - 1, pageParam.getPageSize());
        Page<PersonnelStatistics> all = personnelStatisticsRepository.findAll(
            condition(personnelStatisticsQueryParam),
            pageable
        );
        List<PersonnelStatisticsVo> data = all
            .get()
            .map(personnelStatistics -> {
                PersonnelStatisticsVo vo = new PersonnelStatisticsVo();
                BeanUtils.copyProperties(personnelStatistics, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<PersonnelStatisticsVo> resultVo = PageVo.build(data, all.getTotalElements());
        return Result.ok(resultVo);
    }

    @Override
    public void export(PersonnelStatisticsQueryParam personnelStatisticsQueryParam, HttpServletResponse response) {
        List<PersonnelStatistics> all = personnelStatisticsRepository.findAll(condition(personnelStatisticsQueryParam));
        List<PersonnelStatisticsExportExcel> visitorExportExcels = all
            .stream()
            .map(personnelStatistics -> {
                PersonnelStatisticsExportExcel personnelStatisticsExportExcel = new PersonnelStatisticsExportExcel();
                BeanUtils.copyProperties(personnelStatistics, personnelStatisticsExportExcel);
                return personnelStatisticsExportExcel;
            })
            .toList();

        try (
            DefaultStreamExcelBuilder<PersonnelStatisticsExportExcel> defaultStreamExcelBuilder = DefaultStreamExcelBuilder
                .of(PersonnelStatisticsExportExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(14)
                .widths(8)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            defaultStreamExcelBuilder.append(visitorExportExcels);

            String fileName = URLEncoder.encode("人员统计.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = defaultStreamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("导出访客信息失败: " + e.getMessage());
        }
    }

    public Specification<PersonnelStatistics> condition(PersonnelStatisticsQueryParam personnelStatisticsQueryParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            //查询条件
            if (StrUtil.isNotBlank(personnelStatisticsQueryParam.getRegion())) {
                //区域查询
                list.add(
                    criteriaBuilder.like(
                        root.get(PersonnelStatistics.Fields.region),
                        "%" + personnelStatisticsQueryParam.getRegion() + "%"
                    )
                );
            }
            Order order = criteriaBuilder.desc(root.get(PersonnelStatistics.Fields.updateTime));
            Predicate[] predicates = new Predicate[list.size()];
            query.orderBy(order);
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
